searchNodes=[{"doc":"This module defines an API for creating/manipulating a hash ring. The internal datastructure for the hash ring is actually a gb_tree, which provides fast lookups for a given key on the ring. The ring is a continuum of 2^32 &quot;points&quot;, or integer values Nodes are sharded into 128 points, and distributed across the ring Each shard owns the keyspace below it Keys are hashed and assigned a point on the ring, the node for a given ring is determined by finding the next highest point on the ring for a shard, the node that shard belongs to is then the node which owns that key. If a key&#39;s hash does not have any shards above it, it belongs to the first shard, this mechanism is what creates the ring-like topology. When nodes are added/removed from the ring, only a small subset of keys must be reassigned","ref":"HashRing.html","title":"HashRing","type":"module"},{"doc":"Adds a node to the hash ring, with an optional weight provided which determines the number of virtual nodes (shards) that will be assigned to it on the ring. The default weight for a node is 128 Examples iex&gt; ring = HashRing.new() ...&gt; ring = HashRing.add_node(ring, &quot;a&quot;) ...&gt; %HashRing{nodes: [&quot;b&quot;, &quot;a&quot;]} = ring = HashRing.add_node(ring, &quot;b&quot;, 64) ...&gt; HashRing.key_to_node(ring, :foo) &quot;b&quot;","ref":"HashRing.html#add_node/3","title":"HashRing.add_node/3","type":"function"},{"doc":"Adds a list of nodes to the hash ring. The list can contain just the node key, or a tuple of the node key and it&#39;s desired weight. See also the documentation for add_node/3. Examples iex&gt; ring = HashRing.new() ...&gt; ring = HashRing.add_nodes(ring, [&quot;a&quot;, {&quot;b&quot;, 64}]) ...&gt; %HashRing{nodes: [&quot;b&quot;, &quot;a&quot;]} = ring ...&gt; HashRing.key_to_node(ring, :foo) &quot;b&quot;","ref":"HashRing.html#add_nodes/2","title":"HashRing.add_nodes/2","type":"function"},{"doc":"Determines which node owns the given key. This function assumes that the ring has been populated with at least one node. Examples iex&gt; ring = HashRing.new(&quot;a&quot;) ...&gt; HashRing.key_to_node(ring, :foo) &quot;a&quot; iex&gt; ring = HashRing.new() ...&gt; HashRing.key_to_node(ring, :foo) {:error, {:invalid_ring, :no_nodes}}","ref":"HashRing.html#key_to_node/2","title":"HashRing.key_to_node/2","type":"function"},{"doc":"Determines which nodes owns a given key. Will return either count results or the number of nodes, depending on which is smaller. This function assumes that the ring has been populated with at least one node. Examples iex&gt; ring = HashRing.new() ...&gt; ring = HashRing.add_node(ring, &quot;a&quot;) ...&gt; ring = HashRing.add_node(ring, &quot;b&quot;) ...&gt; ring = HashRing.add_node(ring, &quot;c&quot;) ...&gt; HashRing.key_to_nodes(ring, :foo, 2) [&quot;b&quot;, &quot;c&quot;] iex&gt; ring = HashRing.new() ...&gt; HashRing.key_to_nodes(ring, :foo, 1) {:error, {:invalid_ring, :no_nodes}}","ref":"HashRing.html#key_to_nodes/3","title":"HashRing.key_to_nodes/3","type":"function"},{"doc":"Creates a new hash ring structure, with no nodes added yet Examples iex&gt; ring = HashRing.new() ...&gt; %HashRing{nodes: [&quot;a&quot;]} = ring = HashRing.add_node(ring, &quot;a&quot;) ...&gt; HashRing.key_to_node(ring, {:complex, &quot;key&quot;}) &quot;a&quot;","ref":"HashRing.html#new/0","title":"HashRing.new/0","type":"function"},{"doc":"Creates a new hash ring structure, seeded with the given node, with an optional weight provided which determines the number of virtual nodes (shards) that will be assigned to it on the ring. The default weight for a node is 128 Examples iex&gt; ring = HashRing.new(&quot;a&quot;) ...&gt; %HashRing{nodes: [&quot;a&quot;]} = ring ...&gt; HashRing.key_to_node(ring, :foo) &quot;a&quot; iex&gt; ring = HashRing.new(&quot;a&quot;, 200) ...&gt; %HashRing{nodes: [&quot;a&quot;]} = ring ...&gt; HashRing.key_to_node(ring, :foo) &quot;a&quot;","ref":"HashRing.html#new/2","title":"HashRing.new/2","type":"function"},{"doc":"Returns the list of nodes which are present on the ring. The type of the elements in this list are the same as the type of the elements you initially added to the ring. In the following example, we used strings, but if you were using atoms, such as those used for Erlang node names, you would get a list of atoms back. iex&gt; ring = HashRing.new |&gt; HashRing.add_nodes([&quot;a&quot;, &quot;b&quot;]) ...&gt; HashRing.nodes(ring) [&quot;b&quot;, &quot;a&quot;]","ref":"HashRing.html#nodes/1","title":"HashRing.nodes/1","type":"function"},{"doc":"Removes a node from the hash ring. Examples iex&gt; ring = HashRing.new() ...&gt; %HashRing{nodes: [&quot;a&quot;]} = ring = HashRing.add_node(ring, &quot;a&quot;) ...&gt; %HashRing{nodes: []} = ring = HashRing.remove_node(ring, &quot;a&quot;) ...&gt; HashRing.key_to_node(ring, :foo) {:error, {:invalid_ring, :no_nodes}}","ref":"HashRing.html#remove_node/2","title":"HashRing.remove_node/2","type":"function"},{"doc":"","ref":"HashRing.html#t:t/0","title":"HashRing.t/0","type":"type"},{"doc":"This module defines the API for working with hash rings where the ring state is managed in a GenServer process. There is a performance penalty with working with the ring this way, but it is the best approach if you need to share the ring across multiple processes, or need to maintain multiple rings. If your rings map 1:1 with Erlang node membership, you can configure rings to automatically monitor node up/down events and update the hash ring accordingly, with a default weight, and either whitelist or blacklist nodes from the ring. You configure this at the ring level in your config.exs Each ring is configured in config.exs, and can contain a list of nodes to seed the ring with, and you can then dynamically add/remove nodes to the ring using the API here. Each node on the ring can be configured with a weight, which affects the amount of the total keyspace it owns. The default weight is 128. It&#39;s best to base the weight of nodes on some concrete relative value, such as the amount of memory a node has.","ref":"HashRing.Managed.html","title":"HashRing.Managed","type":"module"},{"doc":"Adds a node to the given hash ring. An error is returned if the ring does not exist, or the node already exists in the ring. Examples iex&gt; {:ok, _pid} = HashRing.Managed.new(:test4) ...&gt; HashRing.Managed.add_node(:test4, &quot;a&quot;) ...&gt; HashRing.Managed.key_to_node(:test4, :foo) &quot;a&quot; iex&gt; HashRing.Managed.add_node(:no_exist, &quot;a&quot;) {:error, :no_such_ring}","ref":"HashRing.Managed.html#add_node/2","title":"HashRing.Managed.add_node/2","type":"function"},{"doc":"Same as add_node/2, but takes a weight value. The weight controls the relative presence this node will have on the ring, the default is 128, but it&#39;s best to give each node a weight value which maps to a concrete resource such as memory or priority. It&#39;s not ideal to have a number which is too high, as it will make the ring datastructure larger, but a good value is probably in the range of 64-256. Examples iex&gt; {:ok, _pid} = HashRing.Managed.new(:test5) ...&gt; HashRing.Managed.add_node(:test5, &quot;a&quot;, 64) ...&gt; HashRing.Managed.key_to_node(:test5, :foo) &quot;a&quot; iex&gt; HashRing.Managed.add_node(:no_exist, &quot;a&quot;) {:error, :no_such_ring}","ref":"HashRing.Managed.html#add_node/3","title":"HashRing.Managed.add_node/3","type":"function"},{"doc":"Adds a list of nodes to the ring. The list of nodes can contain either node names or {node_name, weight} tuples. If there is an error with any of the node weights, an error will be returned, and the ring will remain unchanged. Examples iex&gt; {:ok, _pid} = HashRing.Managed.new(:test6) ...&gt; :ok = HashRing.Managed.add_nodes(:test6, [&quot;a&quot;, {&quot;b&quot;, 64}]) ...&gt; HashRing.Managed.key_to_node(:test6, :foo) &quot;b&quot; iex&gt; {:ok, _pid} = HashRing.Managed.new(:test7) ...&gt; HashRing.Managed.add_nodes(:test7, [&quot;a&quot;, {&quot;b&quot;, :wrong}]) {:error, [{:invalid_weight, &quot;b&quot;, :wrong}]}","ref":"HashRing.Managed.html#add_nodes/2","title":"HashRing.Managed.add_nodes/2","type":"function"},{"doc":"Maps a key to a node on the hash ring. An error is returned if the ring does not exist.","ref":"HashRing.Managed.html#key_to_node/2","title":"HashRing.Managed.key_to_node/2","type":"function"},{"doc":"Creates a new stateful hash ring with the given name. This name is how you will refer to the hash ring via other API calls. It takes an optional set of options which control how the ring behaves. Valid options are as follows: monitor_nodes: boolean: will automatically monitor Erlang node membership, if new nodes are connected or nodes are disconnected, the ring will be updated automatically. In this configuration, nodes cannot be added or removed via the API. Those requests will be ignored. node_blacklist: [String.t | Regex.t]: Used in conjunction with monitor_nodes: true, this is a list of patterns, either as literal strings, or as regex patterns (in either string or literal form), and will be used to ignore nodeup/down events for nodes which are blacklisted. If a node whitelist is provided, the blacklist has no effect. node_whitelist: [String.t | Regex.t]: The same as node_blacklist, except the opposite; only nodes which match a pattern in the whitelist will result in the ring being updated. An error is returned if the ring already exists or if bad ring options are provided. Examples iex&gt; {:ok, _pid} = HashRing.Managed.new(:test1, [nodes: [&quot;a&quot;, {&quot;b&quot;, 64}]]) ...&gt; HashRing.Managed.key_to_node(:test1, :foo) &quot;b&quot; iex&gt; {:ok, pid} = HashRing.Managed.new(:test2) ...&gt; {:error, {:already_started, existing_pid}} = HashRing.Managed.new(:test2) ...&gt; pid == existing_pid true iex&gt; HashRing.Managed.new(:test3, [nodes: &quot;a&quot;]) ** (ArgumentError) {:nodes, &quot;a&quot;} is an invalid option for [`HashRing.Managed.new/2`](HashRing.Managed.html#new/2)","ref":"HashRing.Managed.html#new/2","title":"HashRing.Managed.new/2","type":"function"},{"doc":"Same as HashRing.nodes/1, returns a list of nodes on the ring. Examples iex&gt; {:ok, _pid} = HashRing.Managed.new(:nodes_test) ...&gt; HashRing.Managed.add_nodes(:nodes_test, [:a, :b]) ...&gt; HashRing.Managed.nodes(:nodes_test) [:b, :a]","ref":"HashRing.Managed.html#nodes/1","title":"HashRing.Managed.nodes/1","type":"function"},{"doc":"Removes a node from the given hash ring. An error is returned if the ring does not exist. Examples iex&gt; {:ok, _pid} = HashRing.Managed.new(:test8) ...&gt; :ok = HashRing.Managed.add_nodes(:test8, [&quot;a&quot;, {&quot;b&quot;, 64}]) ...&gt; :ok = HashRing.Managed.remove_node(:test8, &quot;b&quot;) ...&gt; HashRing.Managed.key_to_node(:test8, :foo) &quot;a&quot;","ref":"HashRing.Managed.html#remove_node/2","title":"HashRing.Managed.remove_node/2","type":"function"},{"doc":"","ref":"HashRing.Managed.html#t:key/0","title":"HashRing.Managed.key/0","type":"type"},{"doc":"","ref":"HashRing.Managed.html#t:node_list/0","title":"HashRing.Managed.node_list/0","type":"type"},{"doc":"","ref":"HashRing.Managed.html#t:pattern_list/0","title":"HashRing.Managed.pattern_list/0","type":"type"},{"doc":"","ref":"HashRing.Managed.html#t:ring/0","title":"HashRing.Managed.ring/0","type":"type"},{"doc":"","ref":"HashRing.Managed.html#t:ring_options/0","title":"HashRing.Managed.ring_options/0","type":"type"},{"doc":"","ref":"HashRing.Managed.html#t:weight/0","title":"HashRing.Managed.weight/0","type":"type"},{"doc":"libring - A fast consistent hash ring for Elixir This library implements a stateful consistent hash ring. It&#39;s extremely fast (in benchmarks it&#39;s faster than all other implementations I&#39;ve tested against, namely voicelayer/hash-ring and sile/hash_ring), it has no external dependencies, and is written in Elixir. The algorithm is based on libketama. Nodes on the ring are broken into shards and each one is assigned an integer value in the keyspace, which is the set of integers from 1 to 2^32-1. The distribution of these shards is random, but deterministic. Keys are then mapped to a shard by converting the key to a binary, hashing it with SHA-256, converting the hash to an integer in the keyspace, then finding the shard which is assigned the next highest value, if there is no next highest value, the lowest integer is used, which is how the &quot;ring&quot; is formed. This implementation uses a general balanced tree, via Erlang&#39;s :gb_tree module. Each shard is inserted into the tree, and we use this data structure to efficiently lookup next-highest key and smallest key. I suspect this is why libring is faster than other implementations I&#39;ve benchmarked against.","ref":"readme.html","title":"libring - A fast consistent hash ring for Elixir","type":"extras"},{"doc":"Add :libring to your deps, and run mix deps.get. def deps do [{:libring, &quot;~&gt; 1.0&quot;}] end You have two choices for managing hash rings in your application:","ref":"readme.html#usage","title":"libring - A fast consistent hash ring for Elixir - Usage","type":"extras"},{"doc":"This API works with the raw ring datastructure. It is the fastest implementation, and is best suited for when you have a single process which will need to access the ring, and which can hold the ring in it&#39;s internal state. ring = HashRing.new() |&gt; HashRing.add_node(&quot;a&quot;) |&gt; HashRing.add_node(&quot;b&quot;) &quot;a&quot; = HashRing.key_to_node(ring, {:myworker, 123}) You can also specify the weight of each node, and add nodes in bulk: ring = HashRing.new() |&gt; HashRing.add_nodes([&quot;a&quot;, {&quot;b&quot;, 64}]) |&gt; HashRing.add_node(&quot;c&quot;, 200) &quot;c&quot; = HashRing.key_to_node(ring, {:myworker, 123}) NOTE: Node names do not have to be strings, they can be atoms, tuples, etc.","ref":"readme.html#hashring","title":"libring - A fast consistent hash ring for Elixir - HashRing","type":"extras"},{"doc":"This API works with rings which are held in the internal state of a GenServer process. It supports the same API as HashRing. Because of this, there is a performance overhead due to the messaging, and the GenServer can be a potential bottleneck. If this is the case you are better off exploring ways to use the raw HashRing API. However this API is best suited for situations where you have multiple processes accessing the ring, or need to maintain multiple rings. NOTE: You must have the :libring application started to use this API. {:ok, pid} = HashRing.Managed.new(:myring) :ok = HashRing.Managed.add_node(:myring, &quot;a&quot;) :ok = HashRing.Managed.add_node(:myring, &quot;b&quot;, 64) :ok = HashRing.Managed.add_node(:myring, &quot;c&quot;, 200) &quot;c&quot; = HashRing.Managed.key_to_node(:myring, {:myworker, 123}) You can configure managed rings in config.exs, and they will be created and initialized when the :libring application starts. Configured rings take two shapes, static and dynamic rings. Static rings are simply those where the nodes are provided up front, although you can always add/remove nodes manually at runtime; dynamic rings have Erlang node monitoring enabled, and add or remove nodes on the ring based on cluster membership. You can whitelist/blacklist nodes when using dynamic rings, so that only those nodes which you actually want to distribute work to are used in calculations. This configuration is shown below as well. If you provide a whitelist, the blacklist will have no effect, and only nodes matching the whitelist will be added. If you do not provide a whitelist, the blacklist will be used to filter nodes. If you do not provide either, a default blacklist containing the ~r/^remsh.*$/ pattern from the example below, which is a good default to prevent remote shell sessions (at least those done via releases) from causing the ring to change. The whitelist and blacklist only have an effect when monitor_nodes: true.","ref":"readme.html#hashring-managed","title":"libring - A fast consistent hash ring for Elixir - HashRing.Managed","type":"extras"},{"doc":"Below is an example configuration: config :libring, rings: [ # A ring which automatically changes based on Erlang cluster membership, # but does not allow nodes named &quot;a&quot; or &quot;remsh*&quot; to be added to the ring ring_a: [monitor_nodes: true, node_blacklist: [&quot;a&quot;, ~r/^remsh.*$/]], # A ring which is composed of three nodes, of which &quot;c&quot; has a non-default weight of 200 # The default weight is 128 ring_b: [nodes: [&quot;a&quot;, &quot;b&quot;, {&quot;c&quot;, 200}]] ]","ref":"readme.html#configuration","title":"libring - A fast consistent hash ring for Elixir - Configuration","type":"extras"},{"doc":"To run the test suite you will need to run mix eqc.install --mini once you&#39;ve cloned the repo and fetched dependencies. If you have changes in mind that are significant or potentially time consuming, please open a RFC-style PR first, where we can discuss your plans first. I don&#39;t want you to spend all your time crafting a PR that I ultimately reject because I don&#39;t think it&#39;s a good fit or is too large for me to review. Not that I plan to reject PRs in general, but I have to be careful to balance features with maintenance burden, or I will quickly be unable to manage the project. Please ensure that you adhere to a commit style where logically related changes are in a single commit, or broken up in a way that eases review if necessary. Keep commit subject lines informative, but short, and provide additional detail in the extended message text if needed. If you can, mention relevant issue numbers in either the subject or the extended message.","ref":"readme.html#contributing","title":"libring - A fast consistent hash ring for Elixir - Contributing","type":"extras"},{"doc":"MIT Please see the LICENSE file for more info.","ref":"readme.html#license","title":"libring - A fast consistent hash ring for Elixir - License","type":"extras"}]